#include "menu.hpp"
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
using namespace std;

void Menu::run()
{
    char choice;

    do
    {
        showMainMenu();
        cin >> choice;
        switch (choice)
        {
        case '1':
            addOrder();
            break;
        case '2':
            showOrdersMenu();
            break;
        case '3':
            showStatisticsMenu();
            break;
        }
    }
    while (choice != 'X' && choice != 'x');
}

void Menu::loadData()
{
    _orders.clear(); // ha már vannak rendelések, kitöröljük őket

    ifstream f;
    string row, name;
    stringstream converter;
    int orderNr, cardNr, itemCount, cost;
    char type, ch;

    // adatok betöltése
    f.open("data.dat");
    if (f.good())
    {
        while (!f.eof())
        {
            getline(f, row);
            converter.str(row);
            converter.clear();
            converter >> orderNr >> ch >> itemCount >> ch >> cardNr;
            if (converter.fail()) // ellenőrzés
            {
                f.close();
                return;
            }

            Order newOrder(orderNr, cardNr); // rendelés felvétele

            for (int i = 0; i < itemCount; i++) // rendelés tételei
            {
                getline(f, row);
                converter.str(row);
                converter.clear();
                converter >> type >> ch >> cost >> ch >> name;

                if (converter.fail())
                {
                    f.close();
                    return;
                }

                if (type == 'k' || type == 'n' || type == 't') // tételek felvétele
                    newOrder.addItem(type);
                if (type == 'h' || type == 'u' || type == 'p')
                    newOrder.addItem(type, name, cost);
            }

            _orders.push_back(newOrder);
        }
        f.close();
    }
}
void Menu::saveData() const
{
    ofstream f("data.dat");
    for (int i = 0; i < _orders.size(); i++)
    {
        f << _orders[i].getNumber() << ";"
          << _orders[i].getItemCount() << ";"
          << _orders[i].getCardNumber() << endl;
        for (int j = 0; j < _orders[i].getItemCount(); j++)
        {
            f << _orders[i].getItem(j)->getType() << ";"
              << _orders[i].getItem(j)->getTotalValue() << ";"
              << _orders[i].getItem(j)->getName() << endl;
        }
    }
    f.close();
}

void Menu::showMainMenu() const
{
    cout << endl
         << "Marika néni kávézója" << endl
         << "====================" << endl
         << endl
         << "Kérem válasszon: " << endl
         << "  1: új rendelés" << endl
         << "  2: rendelés lekérdezése" << endl
         << "  3: statisztikák" << endl
         << "  X: kilépés" << endl
         << "Választás: ";
}

void Menu::showStatisticsMenu() const
{
    cout << endl
         << "Statisztikák lekérdezése" << endl
         << "========================" << endl
         << endl
         << "Kérem válasszon: " << endl
         << "  1: összes" << endl
         << "  2: adott törzsvásárlóra" << endl
         << "  X: fõmenü" << endl;
    cout << "Választás: ";

    char choice;
    string input;
    stringstream converter;
    int nr;

    cin >> choice;

    switch (choice)
    {
    case '1':
        showStatisticsForAll();
        break;
    case '2':
        cout << "Törzsvásárlói kártya száma: " << endl;
        cin >> input;
        converter << input;
        if (converter.fail())
            cout << "Hibás bemenet!" << endl;
        else
        {
            converter >> nr;
            showStatisticsForCard(nr);
        }
        break;
    }
}

void Menu::showOrdersMenu() const
{
    cout << endl
         << "Rendelések lekérdezése" << endl
         << "======================" << endl
         << endl
         << "Kérem válasszon: " << endl
         << "  1: adott sorszámra" << endl
         << "  2: adott törzsvásárlóra" << endl
         << "  X: fõmenü" << endl;
    cout << "Választás: ";

    char choice;
    string input;
    stringstream converter;
    int nr;

    cin >> choice;

    switch (choice)
    {
    case '1':
        cout << "Rendelés száma: " << endl;
        cin >> input;
        converter << input;
        if (converter.fail())
            cout << "Hibás bemenet!" << endl;
        else
        {
            converter >> nr;
            showOrder(nr);
        }
        break;
    case '2':
        cout << "Törzsvásárlói kártya száma: " << endl;
        cin >> input;
        converter << input;
        if (converter.fail())
            cout << "Hibás bemenet!" << endl;
        else
        {
            converter >> nr;
            showOrdersForCard(nr);
        }
        break;
    }
}

void Menu::showOrder(int nr) const
{
    if (nr <= 0 || nr > _orders.size())
        cout << "Hibás sorszám!" << endl;

    cout << endl
         << "Sorszám: " << _orders[nr - 1].getNumber() << endl
         << "Kártyaszám: " << _orders[nr - 1].getCardNumber() << endl;
    for (int i = 0; i < _orders[nr - 1].getItemCount(); i++)
    {
        cout << i + 1 << ".: "
             << _orders[nr - 1].getItem(i)->getName() << ", "
             << _orders[nr - 1].getItem(i)->getTotalValue() << " Ft" << endl;
    }
}

void Menu::showOrdersForCard(int nr) const
{
    for (int i = 0; i < _orders.size(); i++)
    {
        cout << endl
             << "Sorszám: " << _orders[i].getNumber() << endl;
        for (int j = 0; j < _orders[i].getItemCount(); j++)
        {
            cout << j + 1 << ".: "
                 << _orders[i].getItem(j)->getName() << ", "
                 << _orders[i].getItem(j)->getTotalValue() << " Ft" << endl;
        }
    }
}

void Menu::showStatisticsForAll() const
{
    int netVal = 0, totalVal = 0;
    for (int i = 0; i < _orders.size(); i++)
    {
        netVal += _orders[i].getNetValue();
        totalVal += _orders[i].getTotalValue();
    }
    cout << "Nettó érték: " << netVal << endl;
    cout << "Bruttó érték: " << totalVal << endl;
}

void Menu::showStatisticsForCard(int cardNr) const
{
    int netVal = 0, totalVal = 0;
    for (int i = 0; i < _orders.size(); i++)
        if (_orders[i].getCardNumber() == cardNr)
        {
            netVal += _orders[i].getNetValue();
            totalVal += _orders[i].getTotalValue();
        }
    cout << "Nettó érték: " << netVal << endl;
    cout << "Bruttó érték: " << totalVal << endl;
}

void Menu::addOrder()
{
    int cardNr, cost;
    char choice;
    string input, name;
    stringstream converter;

    cout << "Törzsvásárlói kártya száma (0, ha nincs): " << endl;
    cin >> input;
    converter << input;
    converter >> cardNr;

    if (converter.fail())
    {
        cout << "Hibás bemenet!" << endl;
        return;
    }

    Order newOrder(_orders.size() + 1, cardNr);

    do
    {
        cout << endl
             << "1: Hamburger hozzáadása" << endl
             << "2: Ufo hozzáadása" << endl
             << "3: palacsinta hozzáadása" << endl
             << "4: tea hozzáadása" << endl
             << "5: narancslé hozzáadása" << endl
             << "6: kóla hozzáadása" << endl
             << "X: fõmenü" << endl
             << "Választás: ";
        cin >> choice;
        switch (choice)
        {
        case '1':
            cout << "Név: ";
            cin >> name;
            cout << "Ár: ";
            cin >> input;
            converter.str(input);
            converter.clear();
            converter >> cost;

            if (converter.fail())
            {
                cout << "Hibás bemenet!" << endl;
                return;
            }

            newOrder.addItem('h', name, cost);
            break;
        case '2':
            cout << "Név: ";
            cin >> name;
            cout << "Ár: ";
            cin >> input;
            converter.str(input);
            converter.clear();
            converter >> cost;

            if (converter.fail())
            {
                cout << "Hibás bemenet!" << endl;
                return;
            }

            newOrder.addItem('u', name, cost);
            break;
        case '3':
            cout << "Név: ";
            cin >> name;
            cout << "Ár: ";
            cin >> input;
            converter.str(input);
            converter.clear();
            converter >> cost;

            if (converter.fail())
            {
                cout << "Hibás bemenet!" << endl;
                return;
            }

            newOrder.addItem('p', name, cost);
            break;
        case '4':
            newOrder.addItem('t');
            break;
        case '5':
            newOrder.addItem('n');
            break;
        case '6':
            newOrder.addItem('k');
            break;
        }
    }
    while (choice != 'X' && choice != 'x');

    _orders.push_back(newOrder);
}
