#ifndef VECTOR_HPP_INCLUDED
#define VECTOR_HPP_INCLUDED

template <class T>
class vector // dinamikus vektor típusa
{
public:
    vector(int size = 0); // konstruktor
    vector(const vector<T>& other); // másoló konstruktor
    ~vector(); // destruktor
    void push_back(T value); // elem beszúrása a vektor végébe
    T pop_back(); // utolsó elem törlése
    int size() const; // méretlekérdezés
    void clear(); // kiürítés

    T& operator[](int index); // indexelõ operátor írásra
    T operator[](int index) const; // indexelõ operátor lekérdezésre

    vector<T>& operator=(const vector<T>& other); // értékadás

    enum Exceptions { SIZE_INVALID, VECTOR_EMPTY, INDEX_OUT_OF_RANGE }; // kivételek
private:
    void resize(int newCapacity); // tömb átméretezése

    T* _values; // értékek tömbjének mutatója
    int _size; // vektor aktuális mérete
    int _capacity; // vektor kapacitása (teljes mérete)
};


template <class T>
vector<T>::vector(int size)
{
    if (size < 0)
        throw SIZE_INVALID;

    if (size > 100) // kezdeti kapacitás beállítása
        _capacity = size * 2;
    else
        _capacity = 100;

    _size = size; // méret átvétele
    _values = new T[_capacity]; // tömb dinamikus létrehozása
}

template <class T>
vector<T>::vector(const vector<T>& other) {
   _capacity = other._capacity; // átmásoljuk az egyszerű értékeket
   _size = other._size;

   _values = new T[_capacity]; // létrehozzuk a dinamikus tömböt

   for (int i = 0; i < _size; i++)
     _values[i] = other._values[i]; // átmásoljuk a hasznos értékeket
}


template <class T>
vector<T>::~vector()
{
    delete[] _values; // dinamikus tömb törlése
}

template <class T>
void vector<T>::push_back(T value)
{
    if (_size == _capacity) // ha már nincs hely
        resize(2 * _capacity); // átméretezés a duplájára

    _values[_size] = value; // behelyezzük az elemet
    _size++; // növeljük az elemszámot
}

template <class T>
T vector<T>::pop_back()
{
    if (_size > 0) // ha van elem
    {
        _size--; // csökkentjük az elemszámot
        return _values[_size]; // visszaadjuk a kivett elemet
    }
    else // különben kivételt dobunk
        throw VECTOR_EMPTY;
}

template <class T>
void vector<T>::clear()
{
    _size = 0;
    resize(100);
}

template <class T>
int vector<T>::size() const
{
    return _size;
}

template <class T>
vector<T>& vector<T>::operator=(const vector<T>& other) {
   if (this == &other) // amennyiben a két memóriacím megegyezik
      return *this; // nem kell semmit csinálni

   delete[] _values; // eddigi dinamikus tömb törlése

   _capacity = other._capacity; // átmásoljuk az egyszerű értékeket
   _size = other._size;

   _values = new T[_capacity]; // létrehozzuk a dinamikus tömböt

   for (int i = 0; i < _size; i++)
     _values[i] = other._values[i]; // átmásoljuk a hasznos értékeket

   return *this;
}

template <class T>
T& vector<T>::operator[](int index)
{
    if (index < 0 || index >= _size) // ellenőrizzük az indexet
        throw INDEX_OUT_OF_RANGE;

    return _values[index];
}

template <class T>
T vector<T>::operator[](int index) const
{
    if (index < 0 || index >= _size)
        throw INDEX_OUT_OF_RANGE;

    return _values[index];
}

template <class T>
void vector<T>::resize(int newCapacity)
{
    T *temp = new T[newCapacity]; // új tömb lefoglalása
    for (int i = 0; i < _size; i++)
    {
        temp[i] = _values[i]; // értékek átmásolása
    }
    delete[] _values; // régi tömb törlése
    _values = temp; // új tömb beállítása
    _capacity = newCapacity; // eltároljuk az új méretet
}

#endif // VECTOR_HPP_INCLUDED
